# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.

import biosteam as bst
import thermosteam as tmo
from . import units

__all__ = ('FattyalcoholProcessSpecifications',)

ReactorSpecification = bst.process_tools.ReactorSpecification

# %% Constant specifications

fermentation_products = ('Hexanol','Octanol', 'Decanol', 
                         'Dodecanol', 'Tetradecanol')

substrates = ('Glucose',)

# %% Process specification

class FattyalcoholProcessSpecifications:
    """
    Create an FattyalcoholProcessSpecifications object that handles overall
    process and technology specifications of the fattyalcohol production system.
    
    """
    __slots__ = ('operating_days',
                 'plant_size',
                 'titer',
                 'productivity',
                 'yield_',
                 'reaction_name',
                 'unit',
    )
    
    products = fermentation_products
    substrates = substrates
    
    def __init__(self, system, tea,
                 plant_size=150000,
                 operating_days=350.4, 
                 titer=80,
                 productivity=1,
                 yield_=0.5):
        self.plant_size = plant_size
        self.operating_days = operating_days
        self.titer = titer
        self.productivity = productivity
        self.yield_ = yield_
        self.reaction_name = 'fermentation_reaction'
        self.unit = tmo.utils.get_instance(system.units, units.FattyAlcoholBioreactor)
        
    def load_specifications(self, plant_size=None, operating_days=None,
                            titer=None, productivity=None, yield_=None):
        self.load_yield(yield_ or self.yield_)
        self.load_operating_days(operating_days or self.operating_days)
        self.load_productivity(productivity or self.productivity)
        self.load_titer(titer or self.titer)
        self.load_plant_size(plant_size or self.plant_size)
        
    def load_plant_size(self, plant_size):
        production = plant_size * 907.18474 / 24. / self.operating_days
        F_mass_substrates = production / self.products_over_substrates
        feed = self.feed
        z_mass_substrates = feed.get_mass_composition(self.substrates).sum()
        feed.F_mass = F_mass_substrates / z_mass_substrates
        self.plant_size = plant_size
    
    def load_operating_days(self, operating_days):
        self.operating_days = self.tea.operating_days = operating_days
    
    load_titer = ReactorSpecification.load_titer
    load_yield = ReactorSpecification.load_yield
    load_productivity = ReactorSpecification.load_productivity
    _calculate_titer = ReactorSpecification._calculate_titer
    _titer_objective_function = ReactorSpecification._titer_objective_function
    evaluate_across_productivity = ReactorSpecification.evaluate_across_productivity
    evaluate_across_productivity = ReactorSpecification.evaluate_across_productivity
    evaluate_across_TRY = ReactorSpecification.evaluate_across_TRY
    feed = ReactorSpecification.feed
    vent = ReactorSpecification.vent
    effluent = ReactorSpecification.effluent
    products_over_substrates = ReactorSpecification.products_over_substrates
    reaction = ReactorSpecification.reaction


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 28 01:25:56 2020

@author: yoelr
"""
from biosteam import BatchBioreactor

__all__ = ('FattyAlcoholBioreactor',)

class FattyAlcoholBioreactor(BatchBioreactor):
    _N_ins = 1
    _N_outs = 2
    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
                 fermentation_reaction, tau,
                 prefermentaion_reaction=None,
                 postfermentation_reaction=None,
                 N=None, V=None, T, P=101325, Nmin=2, Nmax=32):
        BatchBioreactor.__init__(self, ID, ins, outs, thermo, tau=tau,
                                 N=N, V=V, T=T, P=P, Nmin=Nmin, Nmax=Nmax)
        self.prefermentaion_reaction = prefermentaion_reaction
        self.fermentation_reaction = fermentation_reaction
        self.postfermentation_reaction = postfermentation_reaction
        
    @property
    def efficiency(self):
        return self.fermentation_reaction.X
    @efficiency.setter
    def efficiency(self, efficiency):
        self.fermentation_reaction.X = efficiency

    @property
    def Hnet(self):
        return self.fermentation_reaction.dH * self.ins[0].imol[self.fermentation_reaction.reactant]

    def _run(self):
        vent, effluent = self.outs
        effluent.T = self.T
        effluent.P = self.P
        effluent.copy_flow(self.ins[0])
        if self.prefermentaion_reaction: self.prefermentaion_reaction(effluent)
        self.fermentation_reaction(effluent)
        if self.postfermentation_reaction: self.postfermentation_reaction(effluent)
        vent.copy_flow(effluent, 'CO2', remove=True)
        vent.copy_thermal_condition(effluent)
        


# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""
import numpy as np
from biosteam.process_tools import UnitGroup
import pytest

def test_sugarcane():
    from biorefineries import sugarcane as sc
    sc.load()
    units = UnitGroup('Biorefinery', sc.sugarcane_tea.units)
    assert np.allclose(sc.sugarcane_tea.IRR, 0.10319729375184486, rtol=1e-2)
    assert np.allclose(sc.sugarcane_tea.sales, 87641104.25548652, rtol=1e-2)
    assert np.allclose(sc.sugarcane_tea.material_cost, 60096508.050463505, rtol=1e-2)
    assert np.allclose(sc.sugarcane_tea.installed_equipment_cost, 111512561.90122335, rtol=1e-2)
    assert np.allclose(sc.sugarcane_tea.utility_cost, -8589365.38733437, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 272.4872352787688, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 344.88392082847315, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 9.853875285162502, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 37.383892552259844, rtol=1e-2)

def test_lipidcane():
    from biorefineries import lipidcane as lc
    lc.load()
    units = UnitGroup('Biorefinery', lc.lipidcane_tea.units)
    assert np.allclose(lc.lipidcane_tea.IRR, 0.1783634347376759, rtol=1e-2)
    assert np.allclose(lc.lipidcane_tea.sales, 102410581.5597974, rtol=1e-2)
    assert np.allclose(lc.lipidcane_tea.material_cost, 61762971.048900105, rtol=1e-2)
    assert np.allclose(lc.lipidcane_tea.installed_equipment_cost, 139559824.39299855, rtol=1e-2)
    assert np.allclose(lc.lipidcane_tea.utility_cost, -25886609.91411801, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 197.74264093327824, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 305.1670349131174, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 9.65815356491132, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 92.62805713580244, rtol=1e-2)

def test_cornstover():
    from biorefineries import cornstover as cs
    cs.load()
    MESP = cs.cornstover_tea.solve_price(cs.ethanol)
    units = UnitGroup('Biorefinery', cs.cornstover_tea.units)
    assert np.allclose(MESP, 0.7383610895500932, rtol=1e-2)
    assert np.allclose(cs.cornstover_tea.sales, 134055638.87837549, rtol=1e-2)
    assert np.allclose(cs.cornstover_tea.material_cost, 82902092.80482696, rtol=1e-2)
    assert np.allclose(cs.cornstover_tea.installed_equipment_cost, 220012587.48251432, rtol=1e-2)
    assert np.allclose(cs.cornstover_tea.utility_cost, -11047774.698866017, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 318.05635812416523, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 365.67806026618115, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 22.371322764496814, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 45.33827889984683, rtol=1e-2)
    
def test_LAOs():
    from biorefineries import LAOs as laos
    laos.load()
    MPSP = laos.get_LAOs_MPSP()
    units = UnitGroup('Biorefinery', laos.LAOs_tea.units)
    assert np.allclose(MPSP, 1226.0016718824597, rtol=1e-2)
    assert np.allclose(laos.LAOs_tea.sales, 168789574.412942, rtol=1e-2)
    assert np.allclose(laos.LAOs_tea.material_cost, 135661583.58974993, rtol=1e-2)
    assert np.allclose(laos.LAOs_tea.installed_equipment_cost, 76661769.51734665, rtol=0.05)
    assert np.allclose(laos.LAOs_tea.utility_cost, 3811246.803774001, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 59.19501587183199, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 134.66774238880174, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 3.689496361470118, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 3.6894963614701215, rtol=1e-2)   

@pytest.mark.slow
def test_wheatstraw():
    from biorefineries import wheatstraw as ws
    ws.load()
    MESP = ws.wheatstraw_tea.solve_price(ws.ethanol)
    units = UnitGroup('Biorefinery', ws.wheatstraw_tea.units)
    assert np.allclose(MESP, 0.9384667423768497, rtol=1e-2)
    assert np.allclose(ws.wheatstraw_tea.sales, 130275585.21251984, rtol=1e-2)
    assert np.allclose(ws.wheatstraw_tea.material_cost, 63298497.065546915, rtol=1e-2)
    assert np.allclose(ws.wheatstraw_tea.installed_equipment_cost, 242292530.10266995, rtol=1e-2)
    assert np.allclose(ws.wheatstraw_tea.utility_cost, -4373067.719624106, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 219.0919211247056, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 265.2733205865057, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 22.88974837441627, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 31.98081487705149, rtol=1e-2)

@pytest.mark.slow
def test_annimal_bedding():
    from biorefineries import animal_bedding as ab
    ab.load()
    MESP = ab.bedding_tea.solve_price(ab.ethanol)
    units = UnitGroup('Biorefinery', ab.bedding_tea.units)
    assert np.allclose(MESP, 0.8188585523640879, rtol=1e-2)
    assert np.allclose(ab.bedding_tea.sales, 115097527.36582053, rtol=1e-2)
    assert np.allclose(ab.bedding_tea.material_cost, 33081968.8237324, rtol=1e-2)
    assert np.allclose(ab.bedding_tea.installed_equipment_cost, 285002340.76542723, rtol=1e-2)
    assert np.allclose(ab.bedding_tea.utility_cost, -1178035.514879562, rtol=1e-2)
    assert np.allclose(units.get_heating_duty(), 161.56120885595166, rtol=1e-2)
    assert np.allclose(units.get_cooling_duty(), 201.26648298810298, rtol=1e-2)
    assert np.allclose(units.get_electricity_consumption(), 31.999853945401934, rtol=1e-2)
    assert np.allclose(units.get_electricity_production(), 34.44884396679516, rtol=1e-2)
    
if __name__ == '__main__':
    test_sugarcane()
    test_lipidcane()
    test_cornstover()
    test_wheatstraw()
    test_LAOs()
    test_annimal_bedding()